<?php

/**
 * ForteEcom Payment Gateway
 *
 * Provides an ForteEcom Payment Gateway
 *
 * @class          WC_Gateway_ForteEcom
 * @extends        WC_Payment_Gateway
 * @version        1.0.0
 * @package        WooCommerce/Classes/Payment
 * @author         OneTech
 */

class WC_Gateway_ForteEcom extends WC_Payment_Gateway {

    public $testmode;
    public $logging;
    public $merchant_id;
    public $pre_auth;
    public $available_currencies;

    /**
     * Constructor for the gateway.
     */
    public function __construct() {
        $this->id = 'forteecom';
        // $this->icon               = apply_filters('woocommerce_forteecom_icon', '');
        $this->has_fields           = false;
        $this->method_title         = __( 'ForteEcom', 'wc-gateway-forteecom' );
        $this->method_description   = __( 'Pay with ForteEcom', 'wc-gateway-forteecom' );
        $this->supports             = array(
            'products',
            'add_payment_method',
            'tokenization',
            'refunds'
        );
        $this->available_currencies = array( 'KZT' );

        $this->init_form_fields();
        $this->init_settings();

        $this->title       = $this->settings['title'];
        $this->description = $this->settings['description'];
        $this->testmode    = ( $this->settings['testmode'] == 'yes' ) ? true : false;
        $this->logging     = ( $this->settings['logging'] == 'yes' ) ? true : false;
        $this->merchant_id = $this->settings['merchant_id'];
        $this->pre_auth    = ( $this->settings['pre_auth'] == 'yes' ) ? true : false;

        define( 'FORTEECOM_LOGGING_ENABLED', $this->logging ? 1 : 0 );
        define( 'FORTEECOM_DEMO_ENABLED', $this->testmode ? 1 : 0 );

        /**
         * Hooking into WooCommerce actions
         *
         * @see receipt_page
         * @see callback_handler
         * @see forteecom_order_status_changed
         * @see process_admin_options
         */
        add_action( 'woocommerce_receipt_' . $this->id, array( $this, 'receipt_page' ) );
        add_action( 'woocommerce_api_' . strtolower( get_class( $this ) ), array( $this, 'callback_handler' ) );
        add_action( 'woocommerce_order_status_changed', array( $this, 'forteecom_order_status_changed' ) );
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
            $this,
            'process_admin_options'
        ) );

        wp_enqueue_script( 'forteecom-main-script', plugins_url() . "/forteecom/includes/js/main.js", array( 'jquery' ), null, true );
        wp_enqueue_style( 'forteecom-main-style', plugins_url() . "/forteecom/includes/css/main.css" );

        if ( is_add_payment_method_page() ) {
            wp_enqueue_script( 'forteecom-payment_method-script', plugins_url() . "/forteecom/includes/js/payment_method.js", array( 'jquery' ), null, true );
        } elseif ( is_checkout_pay_page() ) {
            wp_enqueue_script( 'forteecom-order-script', plugins_url() . "/forteecom/includes/js/order.js", array( 'jquery' ), null, true );
        }


        if ( ! $this->is_valid_for_use() ) {
            $this->enabled = false;
        }
    }

    /**
     * Initialize Gateway Settings Form Fields
     */
    public function init_form_fields() {
        $this->form_fields = apply_filters( 'wc_forteecom_form_fields', array(

            'enabled' => array(
                'title'   => __( 'Enable/Disable', 'wc-gateway-forteecom' ),
                'type'    => 'checkbox',
                'label'   => __( 'Enable ForteEcom Payment', 'wc-gateway-forteecom' ),
                'default' => 'yes'
            ),

            'merchant_id' => array(
                'title' => __( 'Merchant ID', 'wc-gateway-forteecom' ),
                'type'  => 'text',
            ),

            'pre_auth' => array(
                'title'   => __( 'On-hold state', 'wc-gateway-forteecom' ),
                'type'    => 'checkbox',
                'label'   => __( 'If enabled, payments firstly go to on-hold state. You will need to manually COMPLETE order to complete payment.', 'wc-gateway-forteecom' ),
                'default' => 'no'
            ),

            'title' => array(
                'title'       => __( 'Title', 'wc-gateway-forteecom' ),
                'type'        => 'text',
                'description' => __( 'This controls the title for the payment method the customer sees during checkout.', 'wc-gateway-forteecom' ),
                'default'     => __( 'ForteEcom', 'wc-gateway-forteecom' ),
                'desc_tip'    => true,
            ),

            'description' => array(
                'title'       => __( 'Description', 'wc-gateway-forteecom' ),
                'type'        => 'textarea',
                'description' => __( 'Payment method description that the customer will see on your checkout.', 'wc-gateway-forteecom' ),
                'default'     => __( 'Pay with ForteEcom', 'wc-gateway-forteecom' ),
                'desc_tip'    => true,
            ),

            'testmode' => array(
                'title'       => __( 'Test mode', 'wc-gateway-forteecom' ),
                'type'        => 'checkbox',
                'description' => __( 'Place the payment gateway in development mode.', 'wc-gateway-forteecom' ),
                'default'     => 'yes'
            ),

            'logging' => array(
                'title'       => __( 'Logging', 'wc-gateway-forteecom' ),
                'type'        => 'checkbox',
                'description' => __( 'Log requests and responses to ForteEcom.', 'wc-gateway-forteecom' ),
                'default'     => 'yes'
            ),

        ) );
    }

    /**
     * Checks if currency is KZT and SSL is used
     *
     * @return bool
     */
    function is_valid_for_use() {
        $user_currency = get_option( 'woocommerce_currency' );

        return in_array( $user_currency, $this->available_currencies );
        // TODO: uncomment and replace
        // return (in_array( $user_currency, $this->available_currencies ) && is_ssl());
    }

    /**
     * Callback for "woocommerce_order_status_changed" action
     * Triggered when order status changed. Checks if it was "pre-auth"-ed. If so, sends complete request
     *
     * @param $order_id
     */
    public function forteecom_order_status_changed( $order_id ) {
        $order                   = new WC_Order( $order_id );
        $order_status            = $order->get_status();
        $_forteecom_order_status = $order->get_meta( '_forteecom_order_status' );
        if ( $_forteecom_order_status == ForteEcom_Helper::STATUS_ON_HOLD && in_array( $order_status, array(
                'completed',
                'processing'
            ) ) ) {
            $_forteecom_order_id    = $order->get_meta( '_forteecom_order_id' );
            $_forteecom_session_id  = $order->get_meta( '_forteecom_session_id' );
            $_forteecom_merchant_id = $order->get_meta( '_forteecom_merchant_id' );
            $complete_result        = ForteEcom_Helper::complete_payment( $_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id );

            if ( empty( $complete_result['error_message'] ) ) {
                $order->add_order_note( __( 'Payment status changed from on-hold to complete', 'wc-gateway-forteecom' ) );
                update_post_meta( $order_id, '_forteecom_order_status', ForteEcom_Helper::STATUS_SUCCESS );
            } else {
                $error_message = __( 'Error occurred while changing payment status from on-hold to complete', 'wc-gateway-forteecom' );
                $order->update_status( 'on-hold', $error_message );
                // Notice for admin
                // wc_add_notice( $error_message, 'error' );
            }
        }
    }

    /**
     * Callback for "woocommerce_api_GATEWAY_ID" action which creates endpoint
     * "MERCHANT_HOME_URL/?wc-api=WC_Gateway_ForteEcom" or
     * "MERCHANT_HOME_URL/wc-api/WC_Gateway_ForteEcom" depending on WC version in merchant side
     *
     * @link https://docs.woocommerce.com/document/wc_api-the-woocommerce-api-callback/
     */
    public function callback_handler() {
        // get request data from $_POST or body (json)
        $request = ( ! empty( $_POST['type'] ) ) ? $_POST : json_decode( file_get_contents( "php://input" ), true );
        switch ( $request['type'] ) {
            case 'PaymentComplete':
                $this->handle_webhook_payment_complete( $request['data'] );
                break;
            case 'StatusChanged':
                $this->handle_webhook_status_changed( $request['data'] );
                break;
            case 'CardSaved':
                $this->handle_webhook_save_card( $request['data'] );
                break;
            case 'PaymentRetried':
                $this->handle_webhook_payment_retried( $request['data'] );
                break;
            default:
                break;
        }

        exit;
    }

        /**
     * Handle webhook coming from frontend after payment retry button clicked
     *
     * @param $data
     */
    private function handle_webhook_payment_retried( $data ) {
        // to get the latest order status
        $this->handle_webhook_payment_complete( $data );
        
        $_forteecom_order_id   = wc_clean( $data['orderId'] );
        $_forteecom_session_id = wc_clean( $data['sessionId'] );
        $_forteecom_remote_id  = wc_clean( $data['remoteId'] );
        $order_id              = ForteEcom_Helper::get_order_id_by_forteecom_order_id( $_forteecom_order_id );
        $order                 = wc_get_order( $order_id );
        $total                 = $order->get_total();
        $order_description     = get_bloginfo( 'name' ) . ' - Order: ' . $order_id;

        $order_init_data = [];
        switch ( $order->get_status() ) {
            case 'pending':
                $order_init_data = [
                    "sessionId" => $_forteecom_session_id,
                    "orderId"   => $_forteecom_order_id
                ];
                break;
            case 'failed':
                $order->update_status( 'pending' );
                $init_result = $this->init_payment( $order, $this->merchant_id, $this->pre_auth, $total, $order_id, $order_description, 398, false );

                $order_init_data = [
                    "sessionId" => $init_result['_forteecom_session_id'],
                    "orderId"   => $init_result['_forteecom_order_id']
                ];
                break;
        }

        echo json_encode($order_init_data);
    }

    /**
     * Handle webhook coming from ForteEcom backend after purchase was completed
     *
     * @param $data
     */
    private function handle_webhook_payment_complete( $data ) {
        $_forteecom_order_id   = wc_clean( $data['orderId'] );
        $_forteecom_session_id = wc_clean( $data['sessionId'] );
        $_forteecom_remote_id  = wc_clean( $data['remoteId'] );
        $order_id              = ForteEcom_Helper::get_order_id_by_forteecom_order_id( $_forteecom_order_id );

        if ( $order_id != $_forteecom_remote_id || $order_id == false ) {
            return;
        }

        $order         = wc_get_order( $order_id );
        $status_result = ForteEcom_Helper::get_payment_status( $this->merchant_id, $_forteecom_order_id, $_forteecom_session_id );
        if ( empty( $status_result['error_message'] ) ) {
            ForteEcom_Helper::update_payment_status( $order, $order_id, $_forteecom_order_id, $status_result['_forteecom_order_status'] );
        }

    }

    /**
     * Handle webhook coming from frontend
     *
     * @param $data
     */
    private function handle_webhook_status_changed( $data ) {
        $_forteecom_order_id   = wc_clean( $data['orderId'] );
        $_forteecom_session_id = wc_clean( $data['sessionId'] );
        $_forteecom_remote_id  = wc_clean( $data['remoteId'] );
        $order_id              = ForteEcom_Helper::get_order_id_by_forteecom_order_id( $_forteecom_order_id );

        if ( !$this->testmode && ( $order_id != $_forteecom_remote_id || $order_id == false ) ) {
            return;
        }

        $order         = wc_get_order( $_forteecom_remote_id );
        $status_result = ForteEcom_Helper::get_payment_status( $this->merchant_id, $_forteecom_order_id, $_forteecom_session_id );
        if ( empty( $status_result['error_message'] ) ) {
            ForteEcom_Helper::update_payment_status( $order, $order_id, $_forteecom_order_id, $status_result['_forteecom_order_status'] );
            ForteEcom_Helper::process_payment_status( $status_result['_forteecom_order_status'] );
        } else {
            wc_add_notice( __( 'Error occurred while getting payment status', 'wc-gateway-forteecom' ), 'error' );
        }
        wp_safe_redirect( $order->get_checkout_order_received_url() );

    }

    /**
     * Handle webhook coming from frontend to save credit card info
     */
    private function handle_webhook_save_card( $data ) {
        if ( is_user_logged_in() ) {
            $this->save_credit_card( $data );
        }
    }

    private function save_credit_card( $data ) {
        $card_token        = wc_clean( $data['card_token'] );
        $card_last4        = wc_clean( $data['card_last4'] );
        $card_expiry_month = wc_clean( $data['card_expiry_month'] );
        $card_expiry_year  = wc_clean( $data['card_expiry_year'] );
        $card_type         = wc_clean( $data['card_type'] );

        $existing_tokens = WC_Payment_Tokens::get_tokens( array(
            'gateway_id' => $this->id,
            'token'      => $card_token,
            'user_id'    => get_current_user_id()
        ) );

        foreach ( $existing_tokens as $existing_token ) {
            if ($card_token == $existing_token->get_token()) return;
        }

        $token_cc = new WC_Payment_Token_CC();
        $token_cc->set_token( $card_token );
        $token_cc->set_card_type( $card_type );
        $token_cc->set_expiry_month( $card_expiry_month );
        $token_cc->set_expiry_year( $card_expiry_year );
        $token_cc->set_last4( $card_last4 );
        $token_cc->set_gateway_id( $this->id );
        $token_cc->set_user_id( get_current_user_id() );

        return $token_cc->save();
    }

    /**
     * Show admin options or disable gateway if requirements not meet
     */
    public function admin_options() {
        if ( $this->is_valid_for_use() ) {
            parent::admin_options();
        } else {
            echo '<div class="inline error"><p><strong>' .
                __( 'Gateway Disabled', 'wc-gateway-forteecom' ) .
                '</strong> ' .
                sprintf( __( 'Your site must use secured connection (SSL) and You must use Kazakhstan tenge as your store currency. Fix this and You can use ForteEcom', 'wc-gateway-forteecom' ) ) .
                '</p></div>';
        }
    }

    /**
     * Process the payment and return the result
     * Called when place order button is clicked.
     * If used saved token, purchase immediately otherwise redirect to pay page.
     *
     * @param int $order_id
     *
     * @return array|void
     */
    public function process_payment( $order_id ) {
        $order = wc_get_order( $order_id );

        $payment_token = wc_clean( $_POST['wc-forteecom-payment-token'] );

        if ( empty( $payment_token ) || $payment_token == 'new' ) {
            return array(
                'result'   => 'success',
                'redirect' => $order->get_checkout_payment_url( true )
            );
        } else {

            /** @var WC_Payment_Token_CC $card */
            $card              = WC_Payment_Tokens::get( $payment_token );
            $order_description = get_bloginfo( 'name' ) . ' - Order: ' . $order_id;

            // Security check!
            if ( get_current_user_id() != $card->get_user_id() ) {
                return;
            }

            $init_result = $this->init_payment( $order, $this->merchant_id, $this->pre_auth, $order->get_total(), $order_id, $order_description, 398, false );

            if ( ! empty( $init_result['error_message'] ) ) {
                wc_add_notice( __( 'Initialization process failed on Gateway', 'wc-gateway-forteecom' ), 'error' );

                return array(
                    'result'   => 'fail',
                    'redirect' => '',
                );
            }

            $purchase_result = ForteEcom_Helper::purchase_by_token( $this->merchant_id, $init_result['_forteecom_order_id'], $init_result['_forteecom_session_id'], $order->get_total(), $order_id, $order_description, 398, $card );
            if ( ! empty( $purchase_result['error_message'] ) ) {
                wc_add_notice( $purchase_result['error_message'], 'error' );

                return array(
                    'result'   => 'fail',
                    'redirect' => '',
                );
            }

            ForteEcom_Helper::update_payment_status( $order, $order_id, $init_result['_forteecom_order_id'], $purchase_result['_forteecom_order_status'], true );

            return array(
                'result'   => 'success',
                'redirect' => $this->get_return_url( $order )
            );
        }
    }

    /**
     * Initialize ForteEcom payment
     *
     * @param WC_Order $order
     * @param string $_forteecom_merchant_id
     * @param bool $pre_auth
     * @param float $total
     * @param int $order_id
     * @param string $order_description
     * @param int $currency_id
     * @param bool $print_form
     *
     * @return array
     */
    public function init_payment( $order, $_forteecom_merchant_id, $pre_auth, $total, $order_id, $order_description, $currency_id, $print_form = true ) {
        $init_result = ForteEcom_Helper::init_payment( $_forteecom_merchant_id, $pre_auth, $total, $order_id, $order_description, $currency_id );
        if ( ! empty( $init_result['error_message'] ) ) {
            $order->update_status( 'failed', $init_result['error_message'] );
        } else {
            $this->update_order_forteecom_data(
                $order_id,
                $init_result['_forteecom_order_id'],
                ForteEcom_Helper::STATUS_INIT,
                $init_result['_forteecom_session_id'],
                $this->merchant_id
            );
            $order->add_order_note(
                sprintf(
                    __( 'Forteecom order initialized with order_ID: %s', 'wc-gateway-forteecom' ),
                    $init_result['_forteecom_order_id']
                )
            );
            if ( $print_form ) {
                $this->receipt_page_form(
                    $total,
                    $init_result['_forteecom_order_id'],
                    $init_result['_forteecom_session_id'],
                    $currency_id,
                    $order_id,
                    $order_description
                );
            }
        }

        return $init_result;
    }

    /**
     * Add/update order's ForteEcom data
     *
     * @param int $order_id
     * @param string $_forteecom_order_id
     * @param string $_forteecom_order_status
     * @param string $_forteecom_session_id
     * @param string $_forteecom_merchant_id
     */
    public function update_order_forteecom_data( $order_id, $_forteecom_order_id, $_forteecom_order_status, $_forteecom_session_id, $_forteecom_merchant_id ) {
        update_post_meta( $order_id, '_forteecom_order_id', $_forteecom_order_id );
        update_post_meta( $order_id, '_forteecom_order_status', $_forteecom_order_status );
        update_post_meta( $order_id, '_forteecom_session_id', $_forteecom_session_id );
        update_post_meta( $order_id, '_forteecom_merchant_id', $_forteecom_merchant_id );
    }

    /**
     * Renders ForteEcom widget button and form
     *
     * @param float $total
     * @param string $_forteecom_order_id
     * @param string $_forteecom_session_id
     * @param int $currency
     * @param int $order_id
     * @param string $order_description
     */
    public function receipt_page_form( $total, $_forteecom_order_id, $_forteecom_session_id, $currency, $order_id, $order_description ) {
        $save_card_state = is_user_logged_in() ? 'unchecked' : 'hidden';
        $notify_url      = WC()->api_request_url( get_class( $this ) );
        $mode            = (FORTEECOM_DEMO_ENABLED) ? 'mode="demo"' : '';
        ob_start();
        echo '<p>' . __( 'Thank you for your order, please click the button below to pay with ForteEcom.', 'wc-gateway-forteecom' ) . '</p>';
        echo '<frt-widget-ecom 
                id="forteecom-widget-btn"
                merchant-id="' . $this->merchant_id . '" 
                order-id="' . $_forteecom_order_id . '" 
                session-id="' . $_forteecom_session_id . '" 
                amount="' . $total . '"
                json-class="TransactionSourcePurchase"
                remote-id="' . $order_id . '"
                currency="' . $currency . '"
                description="' . $order_description . '"
                card-save-state="' . $save_card_state . '"
                notify-url="' . $notify_url . '"
                ' . $mode . '
               ></frt-widget-ecom>';

		echo ForteEcom_Helper::get_script();
		echo '<form id="forteecom_checkout_form" method="post" class="woocommerce-checkout" action="' . $notify_url . '">';
		echo '<input type="hidden" name="type" value="StatusChanged" />';
		echo '<input type="hidden" name="data[orderStatus]" value="" id="forteecom_order_status"/>';
		echo '<input type="hidden" name="data[orderId]" value="' . esc_attr( $_forteecom_order_id ) . '" id="forteecom_order_id"/>';
		echo '<input type="hidden" name="data[sessionId]" value="' . esc_attr( $_forteecom_session_id ) . '" id="forteecom_session_id"/>';
		echo '<input type="hidden" name="data[remoteId]" value="' . esc_attr( $order_id ) . '" />';
		wp_nonce_field( 'forteecom-checkout-process', 'forteecom_checkout_process_nonce' );
		echo '</form>';
		echo '<div class="forteecom_overlay">';
		echo '<div class="forteecom_outer">';
		echo '<span>' . __( 'Please, do not close the window', 'wc-gateway-forteecom' ) . '</span>';
		echo '</div>';
		echo '</div>';
		ob_end_flush();
	}

	/**
	 * Callback for "woocommerce_receipt_GATEWAY_ID" action
	 * Renders checkout page
	 *
	 * @param $order_id
	 */
	public function receipt_page( $order_id ) {
		$order                  = wc_get_order( wc_get_order_id_by_order_key( wc_clean( $_GET['key'] ) ) );
		$total                  = $this->get_order_total();
		$order_id               = absint( get_query_var( 'order-pay' ) );
		$order_description      = get_bloginfo( 'name' ) . ' - Order: ' . $order_id;
		$_forteecom_order_id    = $order->get_meta( '_forteecom_order_id' );
		$_forteecom_session_id  = $order->get_meta( '_forteecom_session_id' );
		$_forteecom_merchant_id = $order->get_meta( '_forteecom_merchant_id' );

		switch ( $order->get_status() ) {
			case 'pending':
				if ( empty( $_forteecom_order_id ) && empty( $_forteecom_session_id ) && empty( $_forteecom_merchant_id ) ) {
					$this->init_payment( $order, $this->merchant_id, $this->pre_auth, $total, $order_id, $order_description, 398 );
				} else {
					$status_result = ForteEcom_Helper::get_payment_status( $_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id );
					if ( $status_result['_forteecom_order_status'] == ForteEcom_Helper::STATUS_EXPIRE ) {
						$this->init_payment( $order, $this->merchant_id, $this->pre_auth, $total, $order_id, $order_description, 398 );
					} else {
						$this->receipt_page_form(
							$total,
							$_forteecom_order_id,
							$_forteecom_session_id,
							398,
							$order_id,
							$order_description );
					}
				}
				break;
			case 'failed':
				$order->update_status( 'pending' );
				$this->init_payment( $order, $this->merchant_id, $this->pre_auth, $total, $order_id, $order_description, 398 );
				break;
			case 'on-hold':
				_e( 'Your order is placed and payment is on-hold. Please come back later.', 'wc-gateway-forteecom' );
				break;
		}
	}

	/**
	 * Save credit card info to DB
	 *
	 * @return array
	 */
	public function add_payment_method() {
		return array(
			'result'   => $this->save_credit_card( $_POST['data'] ) ? 'success' : 'failure',
			'redirect' => wc_get_endpoint_url( 'payment-methods' ),
		);
	}

	/**
	 * Payment form on checkout page
	 */
	public function payment_fields() {
		if ( is_add_payment_method_page() ) {
			echo ForteEcom_Helper::get_script();
			echo '<frt-widget-ecom inline="true" merchant-id="' . $this->merchant_id . '"></frt-widget-ecom>';
			echo '<div class="forteecom_overlay">';
			echo '<div class="forteecom_outer">';
			echo '<span>' . __( 'Please, do not close the window', 'wc-gateway-forteecom' ) . '</span>';
			echo '</div>';
			echo '</div>';
		} else {
			echo $this->get_description();
			$this->saved_payment_methods();
		}
	}

	/**
	 * Checkbox for saving credit card info on checkout page
	 */
	public function save_payment_method_checkbox() {
		printf(
			'<p class="form-row woocommerce-SavedPaymentMethods-saveNew">
				<input id="wc-%1$s-new-payment-method" name="wc-%1$s-new-payment-method" type="checkbox" value="true" style="width:auto;" />
				<label for="wc-%1$s-new-payment-method" style="display:inline;">%2$s</label>
			</p>',
            esc_attr( $this->id ),
            __( 'Save payment information to my account for future purchases.', 'wc-gateway-forteecom' )
        );
    }

    /**
     * Delete order's ForteEcom data
     *
     * @param int $order_id
     */
    public function unset_order_forteecom_data( $order_id ) {
        update_post_meta( $order_id, '_forteecom_order_id', null );
        update_post_meta( $order_id, '_forteecom_order_status', null );
        update_post_meta( $order_id, '_forteecom_session_id', null );
        update_post_meta( $order_id, '_forteecom_merchant_id', null );
    }

    public function process_refund($order_id, $amount = null, $reason = '') {
        if (is_null($amount)) $amount = 0;
        $order                  = wc_get_order( $order_id );
        $_forteecom_order_id    = $order->get_meta( '_forteecom_order_id' );
        $_forteecom_session_id  = $order->get_meta( '_forteecom_session_id' );
        $_forteecom_merchant_id = $order->get_meta( '_forteecom_merchant_id' );
        $refund_result = ForteEcom_Helper::refund_payment($_forteecom_merchant_id, $_forteecom_order_id, $_forteecom_session_id, $amount, 398);

        return empty( $refund_result['error_message'] ) ? true : false;
    }

}
